"""
This module contains resolvers for url shorteners. A resolver takes a netloc
(host[:port] and path) and returns a URL as a string.
"""

import socket
import httplib
from functools import wraps

# Currently there is only a single, generic resolver, which should(tm) work
# with all services. One could add more resolvers here, so as to be more
# polite with the url shortening services that provides APIs for developers
# There are probably also some services with a landing page for ads etc, which
# one needs to handle differently

def _io_error_handling(fun):
    """
    Decorator resolvers can use to handle all the standard io/net related
    exceptions
    """

    @wraps(fun)
    def wrapped(*args, **kwargs):
        try:
            return fun(*args, **kwargs)
        except socket.gaierror: # dns error occured.
            return None
    return wrapped


@_io_error_handling
def generic_resolver(netloc, path, timeout=None):
    """
    Generic url fetcher that assumes the target service will response sanely
    to a HEAD request for the url.

    returns the full url or None if the url could not be resolved.
    """
    if timeout:
        conn = httplib.HTTPConnection(netloc, timeout=timeout)
    else:
        conn = httplib.HTTPConnection(netloc)
    conn.request("HEAD", path)
    response = conn.getresponse()
    if response.status in [httplib.TEMPORARY_REDIRECT,
                           httplib.MOVED_PERMANENTLY, httplib.FOUND]:
        return response.getheader("Location") # None by default
    return None



"""Tools for detecting and expanding shortened URLs."""

__version__ = "0.2.4"
__author__ = "Rune Halvorsen <runefh@gmail.com>"
__homepage__ = "http://bitbucket.org/runeh/urlunshort"
__docformat__ = "restructuredtext"

import urlparse
from .resolvers import generic_resolver

# If some tiny url services require a special resolver, this mapping will
# map between hostname -> resolver (currently we have no custom resolvers)
_resolver_map = {

}

"""
.. attribute:: services

List of domains of known URL shortening services.
"""
services = [
    "bit.ly",
    "budurl.com",
    "cli.gs",
    "fa.by",
    "is.gd",
    "lurl.no",
    "moourl.com",
    "smallr.com",
    "snipr.com",
    "snipurl.com",
    "snurl.com",
    "su.pr",
    "tiny.cc",
    "tr.im"]


def resolve(url, timeout=None):
    """
    Resolve shortened URL to a target URL. If the URL could not be resolved,
    return None.

    :argument url: the url to resolve

    :argumet timeout: the max tix to wait for the URL to resolve in seconds
        There's currently no way of telling if a request failed due to
        a timeout or because the URL could not be resolved!

    :returns: the resolved URL. None if URL could not be resolved

    """
    parts = urlparse.urlparse(url)
    resolver = _resolver_map.get(parts.netloc, generic_resolver)
    return resolver(parts.netloc, parts.path, timeout)


def is_shortened(url):
    """Check if the url appears to be shortened, based on the services
    whitelist. **Note:** This will be a best-effort thing, as the list
    if services has to be kept up to date. Also note that valid URLs on
    shortening services (like bit.ly/apidocs) will be assumed to be a
    shortened url.

    :argument url: The URL to check

    :returns: true or false
    """
    parts = urlparse.urlsplit(url)
    if not parts.scheme and not parts.hostname:
        # couldn't parse anything sensible, try again with a scheme.
        parts = urlparse.urlsplit("http://"+url)
        # yes, I dknow about the default_scheme argument to ursplit,
        # and no, it doesn't actully work.

    return bool(parts.hostname in services and parts.path)



import optparse
from urlunshort import resolve

def entrypoint_urlunshort():
    parser = optparse.OptionParser(usage="%prog url")
    options, args = parser.parse_args();
    if args:
        print resolve(args[0])




from setuptools import setup, find_packages
import sys, os

import urlunshort
author, email = urlunshort.__author__[:-1].split(' <')

setup(name='urlunshort',
      version=urlunshort.__version__,
      description=urlunshort.__doc__,
      long_description=open("README").read(),
      classifiers=[
            "Intended Audience :: Developers",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Topic :: Internet :: WWW/HTTP"
            ],
      keywords='',
      author=author,
      author_email=email,
      url=urlunshort.__homepage__,
      license='BSD',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      entry_points={
        'console_scripts': [
            'urlunshort = urlunshort.console:entrypoint_urlunshort',
            ],
        },
      zip_safe=False,
      test_suite = 'nose.collector',
      )
